package com.sun.xml.internal.ws.api.server;

import com.sun.xml.internal.ws.api.SOAPVersion;
import com.sun.xml.internal.ws.api.pipe.ThrowableContainerPropertySet;
import com.sun.xml.internal.ws.wsdl.OperationDispatcher;
import javax.xml.ws.EndpointReference;
import org.w3c.dom.Element;
import com.sun.xml.internal.ws.api.databinding.MetadataReader;
import com.sun.xml.internal.ws.util.xml.XmlUtil;
import java.net.URL;
import com.sun.xml.internal.ws.server.EndpointAwareTube;
import com.sun.xml.internal.ws.api.config.management.EndpointCreationAttributes;
import com.sun.xml.internal.ws.util.ServiceFinder;
import com.sun.xml.internal.ws.api.config.management.ManagedEndpointFactory;
import com.sun.xml.internal.ws.server.EndpointFactory;
import org.xml.sax.EntityResolver;
import java.util.Collection;
import com.sun.xml.internal.ws.api.pipe.ServerTubeAssemblerContext;
import com.sun.org.glassfish.gmbal.ManagedObjectManager;
import com.sun.xml.internal.ws.policy.PolicyMap;
import com.sun.xml.internal.ws.api.model.SEIModel;
import java.util.Iterator;
import java.util.Collections;
import com.sun.xml.internal.ws.api.Component;
import java.util.Set;
import java.util.List;
import com.sun.xml.internal.ws.api.pipe.Engine;
import com.sun.xml.internal.ws.api.pipe.FiberContextSwitchInterceptor;
import com.sun.xml.internal.ws.api.message.Packet;
import java.util.concurrent.Executor;
import com.sun.istack.internal.Nullable;
import com.sun.xml.internal.ws.api.model.wsdl.WSDLPort;
import com.sun.xml.internal.ws.api.WSBinding;
import javax.xml.namespace.QName;
import com.sun.istack.internal.NotNull;
import com.sun.xml.internal.ws.api.pipe.Codec;
import com.sun.xml.internal.ws.api.ComponentRegistry;

public abstract class WSEndpoint<T> implements ComponentRegistry
{
    public WSEndpoint() {
    }
    
    @NotNull
    public abstract Codec createCodec();
    
    @NotNull
    public abstract QName getServiceName();
    
    @NotNull
    public abstract QName getPortName();
    
    @NotNull
    public abstract Class<T> getImplementationClass();
    
    @NotNull
    public abstract WSBinding getBinding();
    
    @NotNull
    public abstract Container getContainer();
    
    @Nullable
    public abstract WSDLPort getPort();
    
    public abstract void setExecutor(@NotNull Executor p0);
    
    public final void schedule(@NotNull Packet request, @NotNull WSEndpoint.CompletionCallback callback) {
        this.schedule(request, callback, null);
    }
    
    public abstract void schedule(@NotNull Packet p0, @NotNull WSEndpoint.CompletionCallback p1, @Nullable FiberContextSwitchInterceptor p2);
    
    public void process(@NotNull Packet request, @NotNull WSEndpoint.CompletionCallback callback, @Nullable FiberContextSwitchInterceptor interceptor) {
        this.schedule(request, callback, interceptor);
    }
    
    public Engine getEngine() {
        throw new UnsupportedOperationException();
    }
    
    @NotNull
    public abstract WSEndpoint.PipeHead createPipeHead();
    
    public abstract void dispose();
    
    @Nullable
    public abstract ServiceDefinition getServiceDefinition();
    
    public List<BoundEndpoint> getBoundEndpoints() {
        Module m = (Module)this.getContainer().getSPI(Module.class);
        return (m != null) ? m.getBoundEndpoints() : null;
    }
    
    @NotNull
    @Deprecated
    public abstract Set<EndpointComponent> getComponentRegistry();
    
    @NotNull
    public Set<Component> getComponents() {
        return Collections.emptySet();
    }
    
    @Nullable
    public <S> S getSPI(@NotNull Class<S> spiType) {
        Set<Component> componentRegistry = this.getComponents();
        if (componentRegistry != null) {
            for (Component c : componentRegistry) {
                S s = c.getSPI(spiType);
                if (s != null) {
                    return s;
                }
                continue;
            }
        }
        return this.getContainer().getSPI(spiType);
    }
    
    @Nullable
    public abstract SEIModel getSEIModel();
    
    @Deprecated
    public abstract PolicyMap getPolicyMap();
    
    @NotNull
    public abstract ManagedObjectManager getManagedObjectManager();
    
    public abstract void closeManagedObjectManager();
    
    @NotNull
    public abstract ServerTubeAssemblerContext getAssemblerContext();
    
    public static <T> WSEndpoint<T> create(@NotNull Class<T> implType, boolean processHandlerAnnotation, @Nullable Invoker invoker, @Nullable QName serviceName, @Nullable QName portName, @Nullable Container container, @Nullable WSBinding binding, @Nullable SDDocumentSource primaryWsdl, @Nullable Collection<? extends SDDocumentSource> metadata, @Nullable EntityResolver resolver, boolean isTransportSynchronous) {
        return create(implType, processHandlerAnnotation, invoker, serviceName, portName, container, binding, primaryWsdl, metadata, resolver, isTransportSynchronous, true);
    }
    
    public static <T> WSEndpoint<T> create(@NotNull Class<T> implType, boolean processHandlerAnnotation, @Nullable Invoker invoker, @Nullable QName serviceName, @Nullable QName portName, @Nullable Container container, @Nullable WSBinding binding, @Nullable SDDocumentSource primaryWsdl, @Nullable Collection<? extends SDDocumentSource> metadata, @Nullable EntityResolver resolver, boolean isTransportSynchronous, boolean isStandard) {
        WSEndpoint<T> endpoint = EndpointFactory.createEndpoint(implType, processHandlerAnnotation, invoker, serviceName, portName, container, binding, primaryWsdl, metadata, resolver, isTransportSynchronous, isStandard);
        Iterator<ManagedEndpointFactory> managementFactories = ServiceFinder.find(ManagedEndpointFactory.class).iterator();
        if (managementFactories.hasNext()) {
            ManagedEndpointFactory managementFactory = (ManagedEndpointFactory)managementFactories.next();
            EndpointCreationAttributes attributes = new EndpointCreationAttributes(processHandlerAnnotation, invoker, resolver, isTransportSynchronous);
            WSEndpoint<T> managedEndpoint = managementFactory.createEndpoint(endpoint, attributes);
            if ((endpoint.getAssemblerContext().getTerminalTube() instanceof EndpointAwareTube)) {
                ((EndpointAwareTube)endpoint.getAssemblerContext().getTerminalTube()).setEndpoint(managedEndpoint);
                return managedEndpoint;
            }
            return managedEndpoint;
        }
        else {
            return endpoint;
        }
    }
    
    @Deprecated
    public static <T> WSEndpoint<T> create(@NotNull Class<T> implType, boolean processHandlerAnnotation, @Nullable Invoker invoker, @Nullable QName serviceName, @Nullable QName portName, @Nullable Container container, @Nullable WSBinding binding, @Nullable SDDocumentSource primaryWsdl, @Nullable Collection<? extends SDDocumentSource> metadata, @Nullable EntityResolver resolver) {
        return create(implType, processHandlerAnnotation, invoker, serviceName, portName, container, binding, primaryWsdl, metadata, resolver, false);
    }
    
    public static <T> WSEndpoint<T> create(@NotNull Class<T> implType, boolean processHandlerAnnotation, @Nullable Invoker invoker, @Nullable QName serviceName, @Nullable QName portName, @Nullable Container container, @Nullable WSBinding binding, @Nullable SDDocumentSource primaryWsdl, @Nullable Collection<? extends SDDocumentSource> metadata, @Nullable URL catalogUrl) {
        return create(implType, processHandlerAnnotation, invoker, serviceName, portName, container, binding, primaryWsdl, metadata, XmlUtil.createEntityResolver(catalogUrl), false);
    }
    
    @NotNull
    public static QName getDefaultServiceName(Class endpointClass) {
        return getDefaultServiceName(endpointClass, true, null);
    }
    
    @NotNull
    public static QName getDefaultServiceName(Class endpointClass, MetadataReader metadataReader) {
        return getDefaultServiceName(endpointClass, true, metadataReader);
    }
    
    @NotNull
    public static QName getDefaultServiceName(Class endpointClass, boolean isStandard) {
        return getDefaultServiceName(endpointClass, isStandard, null);
    }
    
    @NotNull
    public static QName getDefaultServiceName(Class endpointClass, boolean isStandard, MetadataReader metadataReader) {
        return EndpointFactory.getDefaultServiceName(endpointClass, isStandard, metadataReader);
    }
    
    @NotNull
    public static QName getDefaultPortName(@NotNull QName serviceName, Class endpointClass) {
        return getDefaultPortName(serviceName, endpointClass, null);
    }
    
    @NotNull
    public static QName getDefaultPortName(@NotNull QName serviceName, Class endpointClass, MetadataReader metadataReader) {
        return getDefaultPortName(serviceName, endpointClass, true, metadataReader);
    }
    
    @NotNull
    public static QName getDefaultPortName(@NotNull QName serviceName, Class endpointClass, boolean isStandard) {
        return getDefaultPortName(serviceName, endpointClass, isStandard, null);
    }
    
    @NotNull
    public static QName getDefaultPortName(@NotNull QName serviceName, Class endpointClass, boolean isStandard, MetadataReader metadataReader) {
        return EndpointFactory.getDefaultPortName(serviceName, endpointClass, isStandard, metadataReader);
    }
    
    public abstract <T extends EndpointReference> T getEndpointReference(Class<T> p0, String p1, String p2, Element... p3);
    
    public abstract <T extends EndpointReference> T getEndpointReference(Class<T> p0, String p1, String p2, List<Element> p3, List<Element> p4);
    
    public boolean equalsProxiedInstance(WSEndpoint endpoint) {
        return endpoint != null && this.equals(endpoint);
    }
    
    @Nullable
    public abstract OperationDispatcher getOperationDispatcher();
    
    public abstract Packet createServiceResponseForException(ThrowableContainerPropertySet p0, Packet p1, SOAPVersion p2, WSDLPort p3, SEIModel p4, WSBinding p5);
    
    public interface CompletionCallback
    {
        public abstract void onCompletion(@NotNull Packet p0);
    }
    
    public interface PipeHead
    {
        @NotNull
        public abstract Packet process(@NotNull Packet p0, @Nullable WebServiceContextDelegate p1, @Nullable TransportBackChannel p2);
    }
}
